% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

clear all

% dev_l is the device's BHJ layer thickness (m)
global dev_l
dev_l = 135e-9;
% sim_lambda is the simulated wavelength range (nm)
global sim_lambda
sim_lambda = 400:1:900;
% ITO_l is the thickness of the ITO layer (m)
global ITO_l
ITO_l = 110e-9;
% pedot_l is the thickness of the PEDOT:PSS layer (m)
global pedot_l
pedot_l = 30e-9;
% Al_l is the thickness of the aluminum cathode (m)
global Al_l
Al_l = 100e-9;
% Compute carrier generation profile
Gx = TransferMatrix(dev_l*1e9);

% Define genp, generation profile, to be the reverse of Gx in order to make
% data consistent with upcoming BVP calculations.
global genp
genp = Gx(end:-1:1);
% for i = 1:1:length(Gx),
%     genp(i) = Gx(length(Gx)-i+1);
% end

%determine average electric permittivity in the BHJ
global avgperm
avgperm = a_perm;

q = 1.6e-19; %C
KbT = 0.025; %eV

% Intrinsic carrier concentration m^-3
global ni
ni = 1e17;

% Select voltages at which to simulate photocurrent
counter = 1;
for V=0:0.2:0,    
    % Electric field across device, where 1.13 is the difference between
    % the work function of Al and PEDOT:PSS
    global E
    E = (1.13-V)/dev_l;    
%     global Rec
%     Rec = zeros(1,100);
    
%     for recloop = 1:1:1,
        % An arbitrary intial solution is given before computing the set of
        % ODEs in bvp5c.
        solinit = bvpinit(linspace(0,dev_l,10),[1e22 0 1e22 0]);
        
        % Solve for electron and hole carrier concentrations.
        sol = bvp5c(@odeBHJ,@BHJBCs,solinit);
        
        x = linspace(0,dev_l);
        y = deval(sol,x);
        
        % Electron carrier concentration profile
        n = y(1,:);
        
        % Hole carrier concentration profile
        p = y(2,:);
        
        % Plot electron and hole concentration as a function of position
        figure(1)
        semilogy(x*1e9,y(1,:),x*1e9,y(3,:));
        legend('n','p');
        axis([0 dev_l*1e9 1e17 1e26]);
        title('Electron and Hole Concentrations');
        xlabel('depth (nm)');
        ylabel('carrier concentration (m^-^3)');
        
        for i=1:1:100,
            mob_n(i) = un(pos(x(i)));
            mob_p(i) = up(pos(x(i)));
        end
        
        % Compute electric field as a function of position
        k = 1;
        for i = linspace(0,dev_l),
            efield(k) = (avgperm/perm(pos(i)))*E;
            k = k+1;
        end
        
        % Compute electron and hole drift and diffusion current
        l = max(q*mob_n.*y(1,:).*efield+q*KbT*mob_n.*y(2,:)+q*mob_p.*y(3,:).*efield-q*KbT*mob_p.*y(4,:));
        Jn = q*mob_n.*y(1,:).*efield+q*KbT*mob_n.*y(2,:);
        Jn_drift = q*mob_n.*y(1,:).*efield;
        Jn_diff = q*KbT*mob_n.*y(2,:);
        Jp = q*mob_p.*y(3,:).*efield-q*KbT*mob_p.*y(4,:);
        Jp_drift = q*mob_p.*y(3,:).*efield;
        Jp_diff = q*KbT*mob_p.*y(4,:);
        
        % Plot electron and hole current as a function of position
        figure(2)
        plot(x*1e9,q*mob_n.*y(1,:).*efield+q*KbT*mob_n.*y(2,:),x*1e9,q*mob_p.*y(3,:).*efield-q*KbT*mob_p.*y(4,:),x*1e9,q*mob_n.*y(1,:).*efield+q*KbT*mob_n.*y(2,:)+q*mob_p.*y(3,:).*efield-q*KbT*mob_p.*y(4,:));
        J(counter) = l;
        display(l);
        title(['Short circuit current = ',num2str(l),' A/m^2']);
        legend('n','p','total');
        xlabel('depth (nm)');
        ylabel('Current (A/m^2)');
        
        % Compute recombination rate as a function of position
        jk = 1;
        for h = 0:(dev_l/100):dev_l,
            Gen(jk) = Gopt(h);
            Rec(jk) = 0*1e27+(q/(perm(pos(h))*8.85e-12))*(un(pos(h))*up(pos(h))/(un(pos(h))+up(pos(h))))*(y(1,jk).*y(3,jk)-ni^2);
            A(jk) = Gen(jk)-Rec(jk);
            if jk == 100,
                jk = jk;
            else
                jk = jk+1;
            end
        end
        
        % Plot generation and recombination rates as a function of position
        figure(3)
        plot(0:(dev_l/100):(dev_l-(dev_l/100)),A,0:(dev_l/100):(dev_l-(dev_l/100)),Gen,0:(dev_l/100):(dev_l-(dev_l/100)),Rec);
        legend('Net Generation','Photogeneration','Free-carrier Recombination');
        counter = counter+1;
%     end

    % Save recombination rate, electron and hole concentration and current
    % profiles for each simulated applied voltage.
% csvwrite(['[insert file path]\position.csv'],x');
% csvwrite(['[insert file path]\rec_',num2str(V),'V.csv'],Rec');
% csvwrite(['[insert file path]\e_conc_',num2str(V),'V.csv'],abs(y(1,:))');
% csvwrite(['[insert file path]\h_conc_',num2str(V),'V.csv'],abs(y(3,:))');
% csvwrite(['[insert file path]\h_conc_',num2str(V),'V.csv'],Jn');
% csvwrite(['[insert file path]\h_conc_',num2str(V),'V.csv'],Jp');

end

% Export photocurrent as a function of bias
% csvwrite(['[insert file path]\Jph.csv'],J');